function model = ivmInit(model, d)

% IVMINIT Initialise the IVM model.
%
% model = ivmInit(model, d)

% Copyright (c) 2004 Neil D. Lawrence
% File version 
% IVM toolbox version 0.1



if nargin < 2
  d = model.d;
end

% Get number of data.
numData = size(model.y, 1);
numOut = size(model.y, 2);

% Initialise kernel storage.
model.Kstore = zeros(numData, d);

% Initialise Indices
switch model.selectionCriterion
 case 'none'
  model.I = 1:size(model.X, 1);
 otherwise
  model.I = [];
end


% Initialise parameters
model.diagK = kerneldiag(model.X, model.lntheta, model.kernelType);

model.m = sparse(zeros(numData, numOut));
model.beta = sparse(zeros(numData, numOut));

model.varSigma = repmat(model.diagK, 1, numOut);
model.mu = zeros(numData, numOut);

model.g = zeros(numData, numOut);
model.nu = zeros(numData, numOut);

% Initialise site precision and mean
switch model.noiseType  
 
 case {'probit', 'multiprobit'}

  model.u = zeros(numData, numOut);
  model.c = zeros(numData, numOut);

  model = probitUpdateParams(model, 1:numData);
 
 case 'heaviside'
  model.u = zeros(numData, numOut);
  model.c = zeros(numData, numOut);
  model = heavisideUpdateParams(model, 1:numData);

 case 'gaussian'
  
  model.m = model.y;
  model.beta = ones(size(model.m))*1/exp(model.lntheta(3));
  model = gaussianUpdateParams(model, 1:numData);

end

